/*
   Copyright 2009 Last.fm Ltd. 
      - Primarily authored by Max Howell, Jono Cole and Doug Mansell

   This file is part of liblastfm.

   liblastfm is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   liblastfm is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with liblastfm.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef LASTFM_SCROBBLE_POINT_H
#define LASTFM_SCROBBLE_POINT_H

#include "global.h"


namespace lastfm
{

class LASTFM_DLLEXPORT ScrobblePoint
{
public:
    ScrobblePoint();
    ~ScrobblePoint();
    
    /** j is in seconds, and should be 50% the duration of a track */
    explicit ScrobblePoint( uint j );
    ScrobblePoint( const ScrobblePoint& that );
    operator uint() const;
    ScrobblePoint& operator=( const ScrobblePoint& that );

    void setEnforceScrobbleTimeMax( bool enforceScrobbleTimeMax );

    // scrobbles can occur between these two percentages of track duration
    static uint scrobblePercentMin();      //  50
    static uint scrobblePercentMax();      // 100
    static uint defaultScrobblePercent();  //  50

    // Shortest track length allowed to scrobble in seconds
    static uint scrobbleTimeMin();         //  31
    // Upper limit for scrobble time in seconds
    static uint scrobbleTimeMax();         // 240

private:
    class ScrobblePointPrivate * const d;
};

}

#endif
